from oarepo_runtime.records.dumpers.edtf_interval import EDTFIntervalDumperExt


class DocumentsEDTFIntervalDumperExt(EDTFIntervalDumperExt):
    """edtf interval dumper."""

    paths = ["metadata/events/eventDate"]



import marshmallow as ma
from marshmallow import fields as ma_fields
from marshmallow.fields import String
from oarepo_runtime.services.schema.i18n_ui import I18nStrUIField
from oarepo_runtime.services.schema.marshmallow import DictOnlySchema
from oarepo_runtime.services.schema.rdm_ui import RDMCreatorsUISchema
from oarepo_runtime.services.schema.ui import LocalizedEDTFInterval
from oarepo_vocabularies.services.ui_schema import VocabularyI18nStrUIField

from nr_metadata.ui_schema.identifiers import NRObjectIdentifierUISchema


class NREventUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    eventDate = LocalizedEDTFInterval(required=True)

    eventLocation = ma_fields.Nested(lambda: NRLocationUISchema(), required=True)

    eventNameAlternate = ma_fields.List(ma_fields.String())

    eventNameOriginal = ma_fields.String(required=True)


class NRGeoLocationUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    geoLocationPlace = ma_fields.String()

    geoLocationPoint = ma_fields.Nested(lambda: NRGeoLocationPointUISchema())


class NRLocationUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    country = ma_fields.Nested(lambda: NRCountryVocabularyUISchema())

    place = ma_fields.String(required=True)


class NRRelatedItemUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    itemContributors = ma_fields.List(ma_fields.Nested(lambda: RDMCreatorsUISchema()))

    itemCreators = ma_fields.List(ma_fields.Nested(lambda: RDMCreatorsUISchema()))

    itemEndPage = ma_fields.String()

    itemIssue = ma_fields.String()

    itemPIDs = ma_fields.List(ma_fields.Nested(lambda: NRObjectIdentifierUISchema()))

    itemPublisher = ma_fields.String()

    itemRelationType = ma_fields.Nested(lambda: NRItemRelationTypeVocabularyUISchema())

    itemResourceType = ma_fields.Nested(lambda: NRResourceTypeVocabularyUISchema())

    itemStartPage = ma_fields.String()

    itemTitle = ma_fields.String(required=True)

    itemURL = ma_fields.String()

    itemVolume = ma_fields.String()

    itemYear = ma_fields.Integer()


class NRCountryVocabularyUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.INCLUDE

    _id = String(data_key="id", attribute="id")

    _version = String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class NRGeoLocationPointUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    pointLatitude = ma_fields.Float(required=True)

    pointLongitude = ma_fields.Float(required=True)


class NRItemRelationTypeVocabularyUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.INCLUDE

    _id = String(data_key="id", attribute="id")

    _version = String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class NRLanguageVocabularyUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.INCLUDE

    _id = String(data_key="id", attribute="id")

    _version = String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class NRResourceTypeVocabularyUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.INCLUDE

    _id = String(data_key="id", attribute="id")

    _version = String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class NRRightsVocabularyUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.INCLUDE

    _id = String(data_key="id", attribute="id")

    _version = String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class NRSeriesUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    seriesTitle = ma_fields.String(required=True)

    seriesVolume = ma_fields.String()


class NRSubjectCategoryVocabularyUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.INCLUDE

    _id = String(data_key="id", attribute="id")

    _version = String(data_key="@v", attribute="@v")

    title = VocabularyI18nStrUIField()


class NRSubjectUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    classificationCode = ma_fields.String()

    subject = I18nStrUIField()

    subjectScheme = ma_fields.String()

    valueURI = ma_fields.String()


class NRExternalLocationUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    externalLocationNote = ma_fields.String()

    externalLocationURL = ma_fields.String(required=True)





def create_api_blueprint(app):
    """Create DataRecord blueprint."""
    with app.app_context():
        blueprint = app.extensions["nr_metadata.data"].resource_records.as_blueprint()
        blueprint.record_once(init_create_api_blueprint)

        # calls record_once for all other functions starting with "init_addons_"
        # https://stackoverflow.com/questions/58785162/how-can-i-call-function-with-string-value-that-equals-to-function-name
        funcs = globals()
        funcs = [
            v
            for k, v in funcs.items()
            if k.startswith("init_addons_data") and callable(v)
        ]
        for func in funcs:
            blueprint.record_once(func)

    return blueprint


def init_create_api_blueprint(state):
    """Init app."""
    app = state.app
    ext = app.extensions["nr_metadata.data"]

    # register service
    sregistry = app.extensions["invenio-records-resources"].registry
    sregistry.register(
        ext.service_records, service_id=ext.service_records.config.service_id
    )

    # Register indexer
    if hasattr(ext.service_records, "indexer"):
        iregistry = app.extensions["invenio-indexer"].registry
        iregistry.register(
            ext.service_records.indexer,
            indexer_id=ext.service_records.config.service_id,
        )



import re

from flask import current_app
import marshmallow as ma
from invenio_access.permissions import system_identity
from invenio_i18n import gettext as _
from invenio_rdm_records.services.schemas.metadata import CreatorSchema
from invenio_records_resources.proxies import current_service_registry

class RDMNTKCreatorsSchema(CreatorSchema):
    """NTK version of RDM creators schema.

    This version makes sure that organizations are selected from the
    list of organizations in the system. Record will not be valid if
    organization is not in the system.
    """
    
    def _is_organization_exception(self, name):
        """Check if organization name is in exceptions list."""
        return name in current_app.config["NTK_ORGANIZATION_EXCEPTIONS"]

    @ma.validates_schema
    def check_organization_or_affiliation(self, data, **kwargs):
        """Check if organization is in the system."""
        person_or_org = data.get("person_or_org", {})
        if person_or_org.get("type") == "personal":
            affiliations = data.get("affiliations", [])
            for affiliation in affiliations:
                if not self.from_vocabulary(affiliation):
                    raise ma.ValidationError(
                        _(
                            "It is necessary to choose organization from the controlled vocabulary. "
                            "To add organization, please go to "
                            "https://nusl.techlib.cz/cs/migrace-nusl/navrh-novych-hesel/"
                        ),
                        field_name="affiliations",
                    )
        else:
            # organization
            name = person_or_org.get("name")
            affiliations = current_service_registry.get("affiliations")
            found = [
                x["name"]
                for x in affiliations.search(
                    system_identity,
                    q=f'name.suggest:"{escape_opensearch_query(name)}"',
                    size=100,
                ).hits
            ]
            if name not in found and not self._is_organization_exception(name):
                raise ma.ValidationError(
                    _(
                        "It is necessary to choose organization from the controlled vocabulary. "
                        "To add organization, please go to "
                        "https://nusl.techlib.cz/cs/migrace-nusl/navrh-novych-hesel/"
                    ),
                    field_name="person_or_org",
                )
        return data

    def from_vocabulary(self, affiliation):
        """Check if affiliation is from the vocabulary."""
        if "id" not in affiliation:
            return False
        return True
    
def escape_opensearch_query(value: str) -> str:
    """
    Escapes special characters in a string for safe use in OpenSearch query syntax.
    """
    # Escape each special character with a backslash
    escaped = re.sub(r'([\\+\-=&|><!(){}\[\]^"~*?:/])', r"\\\1", value)

    return escaped


from oarepo_runtime.resources.resource import BaseRecordResource


class CommonResource(BaseRecordResource):
    """CommonRecord resource."""

    # here you can for example redefine
    # create_url_rules function to add your own rules



from invenio_records_permissions import RecordPermissionPolicy

# from invenio_records_permissions.generators import SystemProcess, AnyUser


class DocumentsPermissionPolicy(RecordPermissionPolicy):
    """nr_metadata.documents.records.api.DocumentsRecord permissions.
    Values in this class will be merged with permission presets.
    """

    can_search = []
    can_read = []
    can_create = []
    can_update = []
    can_delete = []
    can_manage = []
    can_read_files = []
    can_update_files = []



from invenio_drafts_resources.records.api import DraftRecordIdProviderV2
from invenio_drafts_resources.services.records.components.media_files import (
    MediaFilesAttrConfig,
)
from invenio_rdm_records.records.api import RDMMediaFileRecord, RDMParent, RDMRecord
from invenio_records.systemfields import ConstantField
from invenio_records_resources.records.systemfields import FilesField, IndexField
from invenio_records_resources.records.systemfields.pid import PIDField, PIDFieldContext
from invenio_vocabularies.contrib.affiliations.api import Affiliation
from invenio_vocabularies.contrib.awards.api import Award
from invenio_vocabularies.contrib.funders.api import Funder
from oarepo_runtime.records.pid_providers import UniversalPIDMixin
from oarepo_runtime.records.relations import (
    PIDRelation,
    RelationsField,
    UnstrictPIDRelation,
)
from oarepo_runtime.records.systemfields import (
    FilteredSelector,
    FirstItemSelector,
    MultiSelector,
    PathSelector,
    SyntheticSystemField,
)
from oarepo_vocabularies.records.api import Vocabulary
from oarepo_workflows.records.systemfields.state import (
    RecordStateField,
    RecordStateTimestampField,
)
from oarepo_workflows.records.systemfields.workflow import WorkflowField

from nr_metadata.documents.records.dumpers.dumper import DocumentsDumper
from nr_metadata.documents.records.models import (
    DocumentsMetadata,
    DocumentsParentMetadata,
)
from nr_metadata.records.synthetic_fields import KeywordsFieldSelector


class DocumentsParentRecord(RDMParent):
    model_cls = DocumentsParentMetadata

    workflow = WorkflowField()


class DocumentsIdProvider(UniversalPIDMixin, DraftRecordIdProviderV2):
    pid_type = "dcmnts"


class DocumentsRecord(RDMRecord):

    model_cls = DocumentsMetadata

    schema = ConstantField("$schema", "local://documents-1.0.0.json")

    index = IndexField(
        "documents-documents-1.0.0",
    )

    pid = PIDField(
        provider=DocumentsIdProvider, context_cls=PIDFieldContext, create=True
    )

    dumper = DocumentsDumper()

    people = SyntheticSystemField(
        PathSelector(
            "metadata.creators.person_or_org", "metadata.contributors.person_or_org"
        ),
        filter=lambda x: x.get("type") == "personal",
        map=lambda x: x.get("name"),
        key="syntheticFields.people",
    )

    organizations = SyntheticSystemField(
        MultiSelector(
            FilteredSelector(
                PathSelector(
                    "metadata.creators.person_or_org",
                    "metadata.contributors.person_or_org",
                ),
                filter=lambda x: x["type"] == "personal",
                projection="affiliations.title.cs",
            ),
            FilteredSelector(
                PathSelector(
                    "metadata.creators.person_or_org",
                    "metadata.contributors.person_or_org",
                ),
                filter=lambda x: x["type"] == "organizational",
                projection="name",
            ),
        ),
        key="syntheticFields.organizations",
    )

    keywords = SyntheticSystemField(
        selector=KeywordsFieldSelector("metadata.subjects.subject"),
        key="syntheticFields.keywords",
    )

    date = SyntheticSystemField(
        selector=FirstItemSelector("metadata.dateModified", "metadata.dateIssued"),
        key="syntheticFields.date",
    )

    year = SyntheticSystemField(
        selector=FirstItemSelector(
            "metadata.dateIssued", "metadata.thesis.dateDefended"
        ),
        key="syntheticFields.year",
        filter=lambda x: len(x) >= 4,
        map=lambda x: x[:4],
    )

    defenseYear = SyntheticSystemField(
        selector=PathSelector("metadata.thesis.dateDefended"),
        key="syntheticFields.defenseYear",
        filter=lambda x: len(x) >= 4,
        map=lambda x: x[:4],
    )

    state = RecordStateField(initial="published")

    state_timestamp = RecordStateTimestampField()

    media_files = FilesField(
        key=MediaFilesAttrConfig["_files_attr_key"],
        bucket_id_attr=MediaFilesAttrConfig["_files_bucket_id_attr_key"],
        bucket_attr=MediaFilesAttrConfig["_files_bucket_attr_key"],
        store=False,
        dump=False,
        file_cls=RDMMediaFileRecord,
        create=False,
        delete=False,
    )

    relations = RelationsField(
        affiliations=UnstrictPIDRelation(
            "metadata.contributors.affiliations",
            keys=["name", "id"],
            pid_field=Affiliation.pid,
        ),
        role=PIDRelation(
            "metadata.contributors.role",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("contributor-types"),
        ),
        creators_affiliations=UnstrictPIDRelation(
            "metadata.creators.affiliations",
            keys=["name", "id"],
            pid_field=Affiliation.pid,
        ),
        creators_role=PIDRelation(
            "metadata.creators.role",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("contributor-types"),
        ),
        country=PIDRelation(
            "metadata.events.eventLocation.country",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("countries"),
        ),
        award=UnstrictPIDRelation(
            "metadata.funders.award",
            keys=[
                "title",
                "id",
                "number",
                "program",
                "acronym",
                "identifiers",
                "subjects",
                "organizations",
                "@v",
            ],
            pid_field=Award.pid,
        ),
        funder=UnstrictPIDRelation(
            "metadata.funders.funder",
            keys=["id", "@v", "name", "title"],
            pid_field=Funder.pid,
        ),
        languages=PIDRelation(
            "metadata.languages",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("languages"),
        ),
        itemContributors_affiliations=UnstrictPIDRelation(
            "metadata.relatedItems.itemContributors.affiliations",
            keys=["name", "id"],
            pid_field=Affiliation.pid,
        ),
        itemContributors_role=PIDRelation(
            "metadata.relatedItems.itemContributors.role",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("contributor-types"),
        ),
        itemCreators_affiliations=UnstrictPIDRelation(
            "metadata.relatedItems.itemCreators.affiliations",
            keys=["name", "id"],
            pid_field=Affiliation.pid,
        ),
        itemCreators_role=PIDRelation(
            "metadata.relatedItems.itemCreators.role",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("contributor-types"),
        ),
        itemRelationType=PIDRelation(
            "metadata.relatedItems.itemRelationType",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("item-relation-types"),
        ),
        itemResourceType=PIDRelation(
            "metadata.relatedItems.itemResourceType",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("resource-types"),
        ),
        resourceType=PIDRelation(
            "metadata.resourceType",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("resource-types"),
        ),
        rights=PIDRelation(
            "metadata.rights",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("rights"),
        ),
        subjectCategories=PIDRelation(
            "metadata.subjectCategories",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("subject-categories"),
        ),
        degreeGrantors=PIDRelation(
            "metadata.thesis.degreeGrantors",
            keys=["id", "title", "hierarchy"],
            pid_field=Vocabulary.pid.with_type_ctx("degree-grantors"),
        ),
    )


class RDMRecordMediaFiles(DocumentsRecord):
    """RDM Media file record API."""

    files = FilesField(
        key=MediaFilesAttrConfig["_files_attr_key"],
        bucket_id_attr=MediaFilesAttrConfig["_files_bucket_id_attr_key"],
        bucket_attr=MediaFilesAttrConfig["_files_bucket_attr_key"],
        store=False,
        dump=False,
        file_cls=RDMMediaFileRecord,
        # Don't create
        create=False,
        # Don't delete, we'll manage in the service
        delete=False,
    )


RDMMediaFileRecord.record_cls = RDMRecordMediaFiles



from invenio_drafts_resources.records.api import DraftRecordIdProviderV2
from invenio_drafts_resources.services.records.components.media_files import (
    MediaFilesAttrConfig,
)
from invenio_rdm_records.records.api import RDMMediaFileRecord, RDMParent, RDMRecord
from invenio_records.systemfields import ConstantField
from invenio_records_resources.records.systemfields import FilesField, IndexField
from invenio_records_resources.records.systemfields.pid import PIDField, PIDFieldContext
from invenio_vocabularies.contrib.affiliations.api import Affiliation
from invenio_vocabularies.contrib.awards.api import Award
from invenio_vocabularies.contrib.funders.api import Funder
from oarepo_runtime.records.pid_providers import UniversalPIDMixin
from oarepo_runtime.records.relations import (
    PIDRelation,
    RelationsField,
    UnstrictPIDRelation,
)
from oarepo_vocabularies.records.api import Vocabulary
from oarepo_workflows.records.systemfields.state import (
    RecordStateField,
    RecordStateTimestampField,
)
from oarepo_workflows.records.systemfields.workflow import WorkflowField

from nr_metadata.common.records.dumpers.dumper import CommonDumper
from nr_metadata.common.records.models import CommonMetadata, CommonParentMetadata


class CommonParentRecord(RDMParent):
    model_cls = CommonParentMetadata

    workflow = WorkflowField()


class CommonIdProvider(UniversalPIDMixin, DraftRecordIdProviderV2):
    pid_type = "common"


class CommonRecord(RDMRecord):

    model_cls = CommonMetadata

    schema = ConstantField("$schema", "local://common-1.0.0.json")

    index = IndexField(
        "common-common-1.0.0",
    )

    pid = PIDField(provider=CommonIdProvider, context_cls=PIDFieldContext, create=True)

    dumper = CommonDumper()

    state = RecordStateField(initial="published")

    state_timestamp = RecordStateTimestampField()

    media_files = FilesField(
        key=MediaFilesAttrConfig["_files_attr_key"],
        bucket_id_attr=MediaFilesAttrConfig["_files_bucket_id_attr_key"],
        bucket_attr=MediaFilesAttrConfig["_files_bucket_attr_key"],
        store=False,
        dump=False,
        file_cls=RDMMediaFileRecord,
        create=False,
        delete=False,
    )

    relations = RelationsField(
        affiliations=UnstrictPIDRelation(
            "metadata.contributors.affiliations",
            keys=["name", "id"],
            pid_field=Affiliation.pid,
        ),
        role=PIDRelation(
            "metadata.contributors.role",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("contributor-types"),
        ),
        creators_affiliations=UnstrictPIDRelation(
            "metadata.creators.affiliations",
            keys=["name", "id"],
            pid_field=Affiliation.pid,
        ),
        creators_role=PIDRelation(
            "metadata.creators.role",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("contributor-types"),
        ),
        country=PIDRelation(
            "metadata.events.eventLocation.country",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("countries"),
        ),
        award=UnstrictPIDRelation(
            "metadata.funders.award",
            keys=[
                "title",
                "id",
                "number",
                "program",
                "acronym",
                "identifiers",
                "subjects",
                "organizations",
                "@v",
            ],
            pid_field=Award.pid,
        ),
        funder=UnstrictPIDRelation(
            "metadata.funders.funder",
            keys=["id", "@v", "name", "title"],
            pid_field=Funder.pid,
        ),
        languages=PIDRelation(
            "metadata.languages",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("languages"),
        ),
        itemContributors_affiliations=UnstrictPIDRelation(
            "metadata.relatedItems.itemContributors.affiliations",
            keys=["name", "id"],
            pid_field=Affiliation.pid,
        ),
        itemContributors_role=PIDRelation(
            "metadata.relatedItems.itemContributors.role",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("contributor-types"),
        ),
        itemCreators_affiliations=UnstrictPIDRelation(
            "metadata.relatedItems.itemCreators.affiliations",
            keys=["name", "id"],
            pid_field=Affiliation.pid,
        ),
        itemCreators_role=PIDRelation(
            "metadata.relatedItems.itemCreators.role",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("contributor-types"),
        ),
        itemRelationType=PIDRelation(
            "metadata.relatedItems.itemRelationType",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("item-relation-types"),
        ),
        itemResourceType=PIDRelation(
            "metadata.relatedItems.itemResourceType",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("resource-types"),
        ),
        resourceType=PIDRelation(
            "metadata.resourceType",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("resource-types"),
        ),
        rights=PIDRelation(
            "metadata.rights",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("rights"),
        ),
        subjectCategories=PIDRelation(
            "metadata.subjectCategories",
            keys=["id", "title"],
            pid_field=Vocabulary.pid.with_type_ctx("subject-categories"),
        ),
    )


class RDMRecordMediaFiles(CommonRecord):
    """RDM Media file record API."""

    files = FilesField(
        key=MediaFilesAttrConfig["_files_attr_key"],
        bucket_id_attr=MediaFilesAttrConfig["_files_bucket_id_attr_key"],
        bucket_attr=MediaFilesAttrConfig["_files_bucket_attr_key"],
        store=False,
        dump=False,
        file_cls=RDMMediaFileRecord,
        # Don't create
        create=False,
        # Don't delete, we'll manage in the service
        delete=False,
    )


RDMMediaFileRecord.record_cls = RDMRecordMediaFiles



from invenio_rdm_records.services.config import RDMRecordServiceConfig, _groups_enabled
from invenio_records_resources.services import (
    ConditionalLink,
    LinksTemplate,
    RecordLink,
    pagination_links,
)
from oarepo_runtime.services.components import (
    CustomFieldsComponent,
    process_service_configs,
)
from oarepo_runtime.services.config import (
    has_draft_permission,
    has_permission,
    has_published_record,
    is_published_record,
)
from oarepo_runtime.services.config.service import (
    PermissionsPresetsConfigMixin,
    SearchAllConfigMixin,
)
from oarepo_runtime.services.records import pagination_links_html
from oarepo_workflows.services.components.workflow import WorkflowComponent

from nr_metadata.data.records.api import DataRecord
from nr_metadata.data.services.records.permissions import DataPermissionPolicy
from nr_metadata.data.services.records.results import DataRecordItem, DataRecordList
from nr_metadata.data.services.records.schema import NRDataRecordSchema
from nr_metadata.data.services.records.search import DataSearchOptions


class DataServiceConfig(
    SearchAllConfigMixin, PermissionsPresetsConfigMixin, RDMRecordServiceConfig
):
    """DataRecord service config."""

    result_item_cls = DataRecordItem

    result_list_cls = DataRecordList

    PERMISSIONS_PRESETS = ["workflow"]

    url_prefix = "/nr-metadata-data/"

    base_permission_policy_cls = DataPermissionPolicy

    schema = NRDataRecordSchema

    search = DataSearchOptions

    record_cls = DataRecord

    service_id = "data"
    indexer_queue_name = "data"

    search_item_links_template = LinksTemplate

    @property
    def components(self):
        return process_service_configs(self, CustomFieldsComponent, WorkflowComponent)

    model = "nr_metadata.data"

    @property
    def links_item(self):
        try:
            supercls_links = super().links_item
        except AttributeError:  # if they aren't defined in the superclass
            supercls_links = {}
        links = {
            **supercls_links,
            "access_grants": RecordLink("{+api}/records/{id}/access/grants"),
            "access_groups": RecordLink(
                "{+api}/records/{id}/access/groups", when=_groups_enabled
            ),
            "access_links": RecordLink("{+api}/records/{id}/access/links"),
            "access_users": RecordLink("{+api}/records/{id}/access/users"),
            "draft": RecordLink(
                "{+api}/nr-metadata-data/{id}/draft",
                when=has_draft_permission("read_draft"),
            ),
            "edit_html": RecordLink(
                "{+ui}/nr-metadata-data/{id}/edit",
                when=has_draft_permission("update_draft"),
            ),
            "latest": RecordLink(
                "{+api}/nr-metadata-data/{id}/versions/latest",
                when=has_permission("read"),
            ),
            "latest_html": RecordLink(
                "{+ui}/nr-metadata-data/{id}/latest", when=has_permission("read")
            ),
            "publish": RecordLink(
                "{+api}/nr-metadata-data/{id}/draft/actions/publish",
                when=has_permission("publish"),
            ),
            "record": RecordLink(
                "{+api}/nr-metadata-data/{id}",
                when=has_published_record() & has_permission("read"),
            ),
            "self": ConditionalLink(
                cond=is_published_record(),
                if_=RecordLink(
                    "{+api}/nr-metadata-data/{id}", when=has_permission("read")
                ),
                else_=RecordLink(
                    "{+api}/nr-metadata-data/{id}/draft",
                    when=has_permission("read_draft"),
                ),
            ),
            "self_html": ConditionalLink(
                cond=is_published_record(),
                if_=RecordLink(
                    "{+ui}/nr-metadata-data/{id}", when=has_permission("read")
                ),
                else_=RecordLink(
                    "{+ui}/nr-metadata-data/{id}/preview",
                    when=has_permission("read_draft"),
                ),
            ),
            "versions": RecordLink(
                "{+api}/nr-metadata-data/{id}/versions",
                when=has_permission("search_versions"),
            ),
        }
        return {k: v for k, v in links.items() if v is not None}

    @property
    def links_search_item(self):
        try:
            supercls_links = super().links_search_item
        except AttributeError:  # if they aren't defined in the superclass
            supercls_links = {}
        links = {
            **supercls_links,
            "self": ConditionalLink(
                cond=is_published_record(),
                if_=RecordLink(
                    "{+api}/nr-metadata-data/{id}", when=has_permission("read")
                ),
                else_=RecordLink(
                    "{+api}/nr-metadata-data/{id}/draft",
                    when=has_permission("read_draft"),
                ),
            ),
            "self_html": ConditionalLink(
                cond=is_published_record(),
                if_=RecordLink(
                    "{+ui}/nr-metadata-data/{id}", when=has_permission("read")
                ),
                else_=RecordLink(
                    "{+ui}/nr-metadata-data/{id}/preview",
                    when=has_permission("read_draft"),
                ),
            ),
        }
        return {k: v for k, v in links.items() if v is not None}

    @property
    def links_search(self):
        try:
            supercls_links = super().links_search
        except AttributeError:  # if they aren't defined in the superclass
            supercls_links = {}
        links = {
            **supercls_links,
            **pagination_links("{+api}/nr-metadata-data/{?args*}"),
            **pagination_links_html("{+ui}/nr-metadata-data/{?args*}"),
        }
        return {k: v for k, v in links.items() if v is not None}

    @property
    def links_search_drafts(self):
        try:
            supercls_links = super().links_search_drafts
        except AttributeError:  # if they aren't defined in the superclass
            supercls_links = {}
        links = {
            **supercls_links,
            **pagination_links("{+api}/user/nr-metadata-data/{?args*}"),
            **pagination_links_html("{+ui}/user/nr-metadata-data/{?args*}"),
        }
        return {k: v for k, v in links.items() if v is not None}

    @property
    def links_search_versions(self):
        try:
            supercls_links = super().links_search_versions
        except AttributeError:  # if they aren't defined in the superclass
            supercls_links = {}
        links = {
            **supercls_links,
            **pagination_links("{+api}/nr-metadata-data/{id}/versions{?args*}"),
        }
        return {k: v for k, v in links.items() if v is not None}



from collections import defaultdict

from oarepo_runtime.records.systemfields import PathSelector


class KeywordsFieldSelector(PathSelector):
    def select(self, record):
        ret = super().select(record)
        by_language = defaultdict(list)
        for r in ret:
            lang = r.get("lang", "en")
            value = r.get("value")
            if not value:
                continue
            by_language[lang].append(value)
        return [by_language]



"""Facet definitions."""

from invenio_records_resources.services.records.facets import TermsFacet
from oarepo_runtime.i18n import lazy_gettext as _
from oarepo_runtime.services.facets import MultilingualFacet
from oarepo_runtime.services.facets.date import DateTimeFacet
from oarepo_runtime.services.facets.nested_facet import NestedLabeledFacet
from oarepo_vocabularies.services.facets import VocabularyFacet

record_status = TermsFacet(field="record_status", label=_("record_status"))

has_draft = TermsFacet(field="has_draft", label=_("has_draft"))

expires_at = DateTimeFacet(field="expires_at", label=_("expires_at.label"))

fork_version_id = TermsFacet(field="fork_version_id", label=_("fork_version_id.label"))

access_embargo_active = TermsFacet(
    field="access.embargo.active", label=_("access/embargo/active.label")
)

access_embargo_until = DateTimeFacet(
    field="access.embargo.until", label=_("access/embargo/until.label")
)

access_files = TermsFacet(field="access.files", label=_("access/files.label"))

access_record = TermsFacet(field="access.record", label=_("access/record.label"))

access_status = TermsFacet(field="access.status", label=_("access/status.label"))

metadata_abstract_cs = TermsFacet(
    field="metadata.abstract_cs.keyword", label=_("metadata/abstract.label")
)

metadata_abstract_en = TermsFacet(
    field="metadata.abstract_en.keyword", label=_("metadata/abstract.label")
)

metadata_abstract = MultilingualFacet(
    lang_facets={"cs": metadata_abstract_cs, "en": metadata_abstract_en},
    label=_("metadata/abstract.label"),
)

metadata_abstract_lang = NestedLabeledFacet(
    path="metadata.abstract",
    nested_facet=TermsFacet(
        field="metadata.abstract.lang", label=_("metadata/abstract/lang.label")
    ),
)

metadata_accessibility_cs = TermsFacet(
    field="metadata.accessibility_cs.keyword", label=_("metadata/accessibility.label")
)

metadata_accessibility_en = TermsFacet(
    field="metadata.accessibility_en.keyword", label=_("metadata/accessibility.label")
)

metadata_accessibility = MultilingualFacet(
    lang_facets={"cs": metadata_accessibility_cs, "en": metadata_accessibility_en},
    label=_("metadata/accessibility.label"),
)

metadata_accessibility_lang = NestedLabeledFacet(
    path="metadata.accessibility",
    nested_facet=TermsFacet(
        field="metadata.accessibility.lang",
        label=_("metadata/accessibility/lang.label"),
    ),
)

metadata_additionalTitles_title_cs = TermsFacet(
    field="metadata.additionalTitles.title_cs.keyword",
    label=_("metadata/additionalTitles/title.label"),
)

metadata_additionalTitles_title_en = TermsFacet(
    field="metadata.additionalTitles.title_en.keyword",
    label=_("metadata/additionalTitles/title.label"),
)

metadata_additionalTitles_title = MultilingualFacet(
    lang_facets={
        "cs": metadata_additionalTitles_title_cs,
        "en": metadata_additionalTitles_title_en,
    },
    label=_("metadata/additionalTitles/title.label"),
)

metadata_additionalTitles_title_lang = NestedLabeledFacet(
    path="metadata.additionalTitles.title",
    nested_facet=TermsFacet(
        field="metadata.additionalTitles.title.lang",
        label=_("metadata/additionalTitles/title/lang.label"),
    ),
)

metadata_additionalTitles_titleType = TermsFacet(
    field="metadata.additionalTitles.titleType",
    label=_("metadata/additionalTitles/titleType.label"),
)

metadata_contributors_affiliations = VocabularyFacet(
    field="metadata.contributors.affiliations",
    label=_("metadata/contributors/affiliations.label"),
    vocabulary="affiliations",
)

metadata_contributors_person_or_org_family_name = TermsFacet(
    field="metadata.contributors.person_or_org.family_name",
    label=_("metadata/contributors/person_or_org/family_name.label"),
)

metadata_contributors_person_or_org_given_name = TermsFacet(
    field="metadata.contributors.person_or_org.given_name",
    label=_("metadata/contributors/person_or_org/given_name.label"),
)

metadata_contributors_person_or_org_identifiers_identifier = TermsFacet(
    field="metadata.contributors.person_or_org.identifiers.identifier",
    label=_("metadata/contributors/person_or_org/identifiers/identifier.label"),
)

metadata_contributors_person_or_org_identifiers_scheme = TermsFacet(
    field="metadata.contributors.person_or_org.identifiers.scheme",
    label=_("metadata/contributors/person_or_org/identifiers/scheme.label"),
)

metadata_contributors_person_or_org_name = TermsFacet(
    field="metadata.contributors.person_or_org.name",
    label=_("metadata/contributors/person_or_org/name.label"),
)

metadata_contributors_person_or_org_type = TermsFacet(
    field="metadata.contributors.person_or_org.type",
    label=_("metadata/contributors/person_or_org/type.label"),
)

metadata_contributors_role = VocabularyFacet(
    field="metadata.contributors.role",
    label=_("metadata/contributors/role.label"),
    vocabulary="contributor-types",
)

metadata_creators_affiliations = VocabularyFacet(
    field="metadata.creators.affiliations",
    label=_("metadata/creators/affiliations.label"),
    vocabulary="affiliations",
)

metadata_creators_person_or_org_family_name = TermsFacet(
    field="metadata.creators.person_or_org.family_name",
    label=_("metadata/creators/person_or_org/family_name.label"),
)

metadata_creators_person_or_org_given_name = TermsFacet(
    field="metadata.creators.person_or_org.given_name",
    label=_("metadata/creators/person_or_org/given_name.label"),
)

metadata_creators_person_or_org_identifiers_identifier = TermsFacet(
    field="metadata.creators.person_or_org.identifiers.identifier",
    label=_("metadata/creators/person_or_org/identifiers/identifier.label"),
)

metadata_creators_person_or_org_identifiers_scheme = TermsFacet(
    field="metadata.creators.person_or_org.identifiers.scheme",
    label=_("metadata/creators/person_or_org/identifiers/scheme.label"),
)

metadata_creators_person_or_org_name = TermsFacet(
    field="metadata.creators.person_or_org.name",
    label=_("metadata/creators/person_or_org/name.label"),
)

metadata_creators_person_or_org_type = TermsFacet(
    field="metadata.creators.person_or_org.type",
    label=_("metadata/creators/person_or_org/type.label"),
)

metadata_creators_role = VocabularyFacet(
    field="metadata.creators.role",
    label=_("metadata/creators/role.label"),
    vocabulary="contributor-types",
)

metadata_dateAvailable = DateTimeFacet(
    field="metadata.dateAvailable", label=_("metadata/dateAvailable.label")
)

metadata_dateIssued = DateTimeFacet(
    field="metadata.dateIssued", label=_("metadata/dateIssued.label")
)

metadata_events_eventLocation_country = VocabularyFacet(
    field="metadata.events.eventLocation.country",
    label=_("metadata/events/eventLocation/country.label"),
    vocabulary="countries",
)

metadata_events_eventLocation_place = TermsFacet(
    field="metadata.events.eventLocation.place",
    label=_("metadata/events/eventLocation/place.label"),
)

metadata_funders_award = VocabularyFacet(
    field="metadata.funders.award",
    label=_("metadata/funders/award.label"),
    vocabulary="awards",
)

metadata_funders_funder = VocabularyFacet(
    field="metadata.funders.funder",
    label=_("metadata/funders/funder.label"),
    vocabulary="funders",
)

metadata_geoLocations_geoLocationPlace = TermsFacet(
    field="metadata.geoLocations.geoLocationPlace",
    label=_("metadata/geoLocations/geoLocationPlace.label"),
)

metadata_geoLocations_geoLocationPoint_pointLatitude = TermsFacet(
    field="metadata.geoLocations.geoLocationPoint.pointLatitude",
    label=_("metadata/geoLocations/geoLocationPoint/pointLatitude.label"),
)

metadata_geoLocations_geoLocationPoint_pointLongitude = TermsFacet(
    field="metadata.geoLocations.geoLocationPoint.pointLongitude",
    label=_("metadata/geoLocations/geoLocationPoint/pointLongitude.label"),
)

metadata_languages = VocabularyFacet(
    field="metadata.languages",
    label=_("metadata/languages.label"),
    vocabulary="languages",
)

metadata_methods_cs = TermsFacet(
    field="metadata.methods_cs.keyword", label=_("metadata/methods.label")
)

metadata_methods_en = TermsFacet(
    field="metadata.methods_en.keyword", label=_("metadata/methods.label")
)

metadata_methods = MultilingualFacet(
    lang_facets={"cs": metadata_methods_cs, "en": metadata_methods_en},
    label=_("metadata/methods.label"),
)

metadata_methods_lang = NestedLabeledFacet(
    path="metadata.methods",
    nested_facet=TermsFacet(
        field="metadata.methods.lang", label=_("metadata/methods/lang.label")
    ),
)

metadata_objectIdentifiers_identifier = TermsFacet(
    field="metadata.objectIdentifiers.identifier",
    label=_("metadata/objectIdentifiers/identifier.label"),
)

metadata_objectIdentifiers_scheme = TermsFacet(
    field="metadata.objectIdentifiers.scheme",
    label=_("metadata/objectIdentifiers/scheme.label"),
)

metadata_originalRecord = TermsFacet(
    field="metadata.originalRecord", label=_("metadata/originalRecord.label")
)

metadata_relatedItems_itemContributors_affiliations = VocabularyFacet(
    field="metadata.relatedItems.itemContributors.affiliations",
    label=_("metadata/relatedItems/itemContributors/affiliations.label"),
    vocabulary="affiliations",
)

metadata_relatedItems_itemContributors_person_or_org_family_name = TermsFacet(
    field="metadata.relatedItems.itemContributors.person_or_org.family_name",
    label=_("metadata/relatedItems/itemContributors/person_or_org/family_name.label"),
)

metadata_relatedItems_itemContributors_person_or_org_given_name = TermsFacet(
    field="metadata.relatedItems.itemContributors.person_or_org.given_name",
    label=_("metadata/relatedItems/itemContributors/person_or_org/given_name.label"),
)

metadata_relatedItems_itemContributors_person_or_org_identifiers_identifier = TermsFacet(
    field="metadata.relatedItems.itemContributors.person_or_org.identifiers.identifier",
    label=_(
        "metadata/relatedItems/itemContributors/person_or_org/identifiers/identifier.label"
    ),
)

metadata_relatedItems_itemContributors_person_or_org_identifiers_scheme = TermsFacet(
    field="metadata.relatedItems.itemContributors.person_or_org.identifiers.scheme",
    label=_(
        "metadata/relatedItems/itemContributors/person_or_org/identifiers/scheme.label"
    ),
)

metadata_relatedItems_itemContributors_person_or_org_name = TermsFacet(
    field="metadata.relatedItems.itemContributors.person_or_org.name",
    label=_("metadata/relatedItems/itemContributors/person_or_org/name.label"),
)

metadata_relatedItems_itemContributors_person_or_org_type = TermsFacet(
    field="metadata.relatedItems.itemContributors.person_or_org.type",
    label=_("metadata/relatedItems/itemContributors/person_or_org/type.label"),
)

metadata_relatedItems_itemContributors_role = VocabularyFacet(
    field="metadata.relatedItems.itemContributors.role",
    label=_("metadata/relatedItems/itemContributors/role.label"),
    vocabulary="contributor-types",
)

metadata_relatedItems_itemCreators_affiliations = VocabularyFacet(
    field="metadata.relatedItems.itemCreators.affiliations",
    label=_("metadata/relatedItems/itemCreators/affiliations.label"),
    vocabulary="affiliations",
)

metadata_relatedItems_itemCreators_person_or_org_family_name = TermsFacet(
    field="metadata.relatedItems.itemCreators.person_or_org.family_name",
    label=_("metadata/relatedItems/itemCreators/person_or_org/family_name.label"),
)

metadata_relatedItems_itemCreators_person_or_org_given_name = TermsFacet(
    field="metadata.relatedItems.itemCreators.person_or_org.given_name",
    label=_("metadata/relatedItems/itemCreators/person_or_org/given_name.label"),
)

metadata_relatedItems_itemCreators_person_or_org_identifiers_identifier = TermsFacet(
    field="metadata.relatedItems.itemCreators.person_or_org.identifiers.identifier",
    label=_(
        "metadata/relatedItems/itemCreators/person_or_org/identifiers/identifier.label"
    ),
)

metadata_relatedItems_itemCreators_person_or_org_identifiers_scheme = TermsFacet(
    field="metadata.relatedItems.itemCreators.person_or_org.identifiers.scheme",
    label=_(
        "metadata/relatedItems/itemCreators/person_or_org/identifiers/scheme.label"
    ),
)

metadata_relatedItems_itemCreators_person_or_org_name = TermsFacet(
    field="metadata.relatedItems.itemCreators.person_or_org.name",
    label=_("metadata/relatedItems/itemCreators/person_or_org/name.label"),
)

metadata_relatedItems_itemCreators_person_or_org_type = TermsFacet(
    field="metadata.relatedItems.itemCreators.person_or_org.type",
    label=_("metadata/relatedItems/itemCreators/person_or_org/type.label"),
)

metadata_relatedItems_itemCreators_role = VocabularyFacet(
    field="metadata.relatedItems.itemCreators.role",
    label=_("metadata/relatedItems/itemCreators/role.label"),
    vocabulary="contributor-types",
)

metadata_relatedItems_itemEndPage = TermsFacet(
    field="metadata.relatedItems.itemEndPage",
    label=_("metadata/relatedItems/itemEndPage.label"),
)

metadata_relatedItems_itemIssue = TermsFacet(
    field="metadata.relatedItems.itemIssue",
    label=_("metadata/relatedItems/itemIssue.label"),
)

metadata_relatedItems_itemPIDs_identifier = TermsFacet(
    field="metadata.relatedItems.itemPIDs.identifier",
    label=_("metadata/relatedItems/itemPIDs/identifier.label"),
)

metadata_relatedItems_itemPIDs_scheme = TermsFacet(
    field="metadata.relatedItems.itemPIDs.scheme",
    label=_("metadata/relatedItems/itemPIDs/scheme.label"),
)

metadata_relatedItems_itemPublisher = TermsFacet(
    field="metadata.relatedItems.itemPublisher",
    label=_("metadata/relatedItems/itemPublisher.label"),
)

metadata_relatedItems_itemRelationType = VocabularyFacet(
    field="metadata.relatedItems.itemRelationType",
    label=_("metadata/relatedItems/itemRelationType.label"),
    vocabulary="item-relation-types",
)

metadata_relatedItems_itemResourceType = VocabularyFacet(
    field="metadata.relatedItems.itemResourceType",
    label=_("metadata/relatedItems/itemResourceType.label"),
    vocabulary="resource-types",
)

metadata_relatedItems_itemStartPage = TermsFacet(
    field="metadata.relatedItems.itemStartPage",
    label=_("metadata/relatedItems/itemStartPage.label"),
)

metadata_relatedItems_itemURL = TermsFacet(
    field="metadata.relatedItems.itemURL",
    label=_("metadata/relatedItems/itemURL.label"),
)

metadata_relatedItems_itemVolume = TermsFacet(
    field="metadata.relatedItems.itemVolume",
    label=_("metadata/relatedItems/itemVolume.label"),
)

metadata_relatedItems_itemYear = TermsFacet(
    field="metadata.relatedItems.itemYear",
    label=_("metadata/relatedItems/itemYear.label"),
)

metadata_resourceType = VocabularyFacet(
    field="metadata.resourceType",
    label=_("metadata/resourceType.label"),
    vocabulary="resource-types",
)

metadata_rights = VocabularyFacet(
    field="metadata.rights", label=_("metadata/rights.label"), vocabulary="rights"
)

metadata_series_seriesTitle = TermsFacet(
    field="metadata.series.seriesTitle", label=_("metadata/series/seriesTitle.label")
)

metadata_series_seriesVolume = TermsFacet(
    field="metadata.series.seriesVolume", label=_("metadata/series/seriesVolume.label")
)

metadata_subjectCategories = VocabularyFacet(
    field="metadata.subjectCategories",
    label=_("metadata/subjectCategories.label"),
    vocabulary="subject-categories",
)

metadata_subjects_classificationCode = TermsFacet(
    field="metadata.subjects.classificationCode",
    label=_("metadata/subjects/classificationCode.label"),
)

metadata_subjects_subject_cs = TermsFacet(
    field="metadata.subjects.subject_cs.keyword",
    label=_("metadata/subjects/subject.label"),
)

metadata_subjects_subject_en = TermsFacet(
    field="metadata.subjects.subject_en.keyword",
    label=_("metadata/subjects/subject.label"),
)

metadata_subjects_subject = MultilingualFacet(
    lang_facets={
        "cs": metadata_subjects_subject_cs,
        "en": metadata_subjects_subject_en,
    },
    label=_("metadata/subjects/subject.label"),
)

metadata_subjects_subject_lang = NestedLabeledFacet(
    path="metadata.subjects.subject",
    nested_facet=TermsFacet(
        field="metadata.subjects.subject.lang",
        label=_("metadata/subjects/subject/lang.label"),
    ),
)

metadata_subjects_subjectScheme = TermsFacet(
    field="metadata.subjects.subjectScheme",
    label=_("metadata/subjects/subjectScheme.label"),
)

metadata_subjects_valueURI = TermsFacet(
    field="metadata.subjects.valueURI", label=_("metadata/subjects/valueURI.label")
)

metadata_systemIdentifiers_identifier = TermsFacet(
    field="metadata.systemIdentifiers.identifier",
    label=_("metadata/systemIdentifiers/identifier.label"),
)

metadata_systemIdentifiers_scheme = TermsFacet(
    field="metadata.systemIdentifiers.scheme",
    label=_("metadata/systemIdentifiers/scheme.label"),
)

metadata_technicalInfo_cs = TermsFacet(
    field="metadata.technicalInfo_cs.keyword", label=_("metadata/technicalInfo.label")
)

metadata_technicalInfo_en = TermsFacet(
    field="metadata.technicalInfo_en.keyword", label=_("metadata/technicalInfo.label")
)

metadata_technicalInfo = MultilingualFacet(
    lang_facets={"cs": metadata_technicalInfo_cs, "en": metadata_technicalInfo_en},
    label=_("metadata/technicalInfo.label"),
)

metadata_technicalInfo_lang = NestedLabeledFacet(
    path="metadata.technicalInfo",
    nested_facet=TermsFacet(
        field="metadata.technicalInfo.lang",
        label=_("metadata/technicalInfo/lang.label"),
    ),
)

metadata_version = TermsFacet(
    field="metadata.version", label=_("metadata/version.label")
)

state = TermsFacet(field="state", label=_("state.label"))

state_timestamp = DateTimeFacet(
    field="state_timestamp", label=_("state_timestamp.label")
)



from oarepo_runtime.resources.resource import BaseRecordResource


class DataciteResource(BaseRecordResource):
    """DataciteRecord resource."""

    # here you can for example redefine
    # create_url_rules function to add your own rules



from oarepo_runtime.records.dumpers.multilingual_dumper import MultilingualDumper


class MultilingualSearchDumperExt(MultilingualDumper):
    """Multilingual search dumper."""

    paths = [
        "metadata/abstract",
        "metadata/accessibility",
        "metadata/additionalTitles/title",
        "metadata/methods",
        "metadata/subjects/subject",
        "metadata/technicalInfo",
    ]
    SUPPORTED_LANGS = ["cs", "en"]

    def dump(self, record, data):
        super().dump(record, data)

    def load(self, record, data):
        super().load(record, data)



from nr_metadata.datacite.records.api import DataciteRecord
from nr_metadata.datacite.resources.records.config import DataciteResourceConfig
from nr_metadata.datacite.resources.records.resource import DataciteResource
from nr_metadata.datacite.services.records.config import DataciteServiceConfig
from nr_metadata.datacite.services.records.service import DataciteService

DATACITE_RECORD_RESOURCE_CONFIG = DataciteResourceConfig


DATACITE_RECORD_RESOURCE_CLASS = DataciteResource


DATACITE_RECORD_SERVICE_CONFIG = DataciteServiceConfig


DATACITE_RECORD_SERVICE_CLASS = DataciteService


OAREPO_PRIMARY_RECORD_SERVICE = {DataciteRecord: "datacite"}



import marshmallow as ma
from edtf import Date as EDTFDate
from invenio_rdm_records.services.schemas.access import AccessSchema
from invenio_rdm_records.services.schemas.pids import PIDSchema
from invenio_rdm_records.services.schemas.record import validate_scheme
from invenio_vocabularies.services.schema import i18n_strings
from marshmallow import fields as ma_fields
from marshmallow.fields import Dict, Nested, String
from marshmallow_utils.fields import SanitizedUnicode, TrimmedString
from marshmallow_utils.fields.nestedattr import NestedAttribute
from oarepo_runtime.services.schema.marshmallow import (
    DictOnlySchema,
    RDMBaseRecordSchema,
)
from oarepo_runtime.services.schema.rdm import FundingSchema
from oarepo_runtime.services.schema.validation import (
    CachedMultilayerEDTFValidator,
    validate_datetime,
    validate_identifier,
)
from oarepo_vocabularies.services.schema import HierarchySchema
from oarepo_workflows.services.records.schema import RDMWorkflowParentSchema

from nr_metadata.common.services.records.schema_common import (
    AdditionalTitlesSchema,
    NRCommonMetadataSchema,
)
from nr_metadata.common.services.records.schema_datatypes import (
    NREventSchema,
    NRExternalLocationSchema,
    NRGeoLocationSchema,
    NRRelatedItemSchema,
    NRSeriesSchema,
    NRSubjectSchema,
)
from nr_metadata.schema.creators import RDMNTKCreatorsSchema
from nr_metadata.schema.identifiers import (
    NRObjectIdentifierSchema,
    NRSystemIdentifierSchema,
)


class GeneratedParentSchema(RDMWorkflowParentSchema):
    """"""

    owners = ma.fields.List(ma.fields.Dict(), load_only=True)


class NRDocumentRecordSchema(RDMBaseRecordSchema):
    class Meta:
        unknown = ma.RAISE

    access = NestedAttribute(lambda: AccessSchema())

    metadata = ma_fields.Nested(lambda: NRDocumentMetadataSchema())

    pids = Dict(
        keys=SanitizedUnicode(validate=validate_scheme),
        values=Nested(PIDSchema),
    )

    state = ma_fields.String(dump_only=True)

    state_timestamp = ma_fields.String(dump_only=True, validate=[validate_datetime])

    syntheticFields = ma_fields.Nested(lambda: NRDocumentSyntheticFieldsSchema())
    parent = ma.fields.Nested(GeneratedParentSchema)


class NRDocumentMetadataSchema(NRCommonMetadataSchema):
    class Meta:
        unknown = ma.RAISE

    additionalTitles = ma_fields.List(
        ma_fields.Nested(lambda: AdditionalTitlesSchema())
    )

    contributors = ma_fields.List(ma_fields.Nested(lambda: RDMNTKCreatorsSchema()))

    creators = ma_fields.List(
        ma_fields.Nested(lambda: RDMNTKCreatorsSchema()),
        required=True,
        validate=[ma.validate.Length(min=1)],
    )

    dateModified = TrimmedString(
        validate=[CachedMultilayerEDTFValidator(types=(EDTFDate,))]
    )

    events = ma_fields.List(ma_fields.Nested(lambda: NREventSchema()))

    externalLocation = ma_fields.Nested(lambda: NRExternalLocationSchema())

    funders = ma_fields.List(ma_fields.Nested(lambda: FundingSchema()))

    geoLocations = ma_fields.List(ma_fields.Nested(lambda: NRGeoLocationSchema()))

    objectIdentifiers = ma_fields.List(
        ma_fields.Nested(
            lambda: NRObjectIdentifierSchema(),
            validate=[lambda value: validate_identifier(value)],
        )
    )

    publishers = ma_fields.List(ma_fields.String())

    relatedItems = ma_fields.List(ma_fields.Nested(lambda: NRRelatedItemSchema()))

    series = ma_fields.List(ma_fields.Nested(lambda: NRSeriesSchema()))

    subjects = ma_fields.List(ma_fields.Nested(lambda: NRSubjectSchema()))

    systemIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: NRSystemIdentifierSchema())
    )

    thesis = ma_fields.Nested(lambda: NRThesisSchema())


class NRThesisSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    dateDefended = TrimmedString(
        validate=[CachedMultilayerEDTFValidator(types=(EDTFDate,))]
    )

    defended = ma_fields.Boolean()

    degreeGrantors = ma_fields.List(ma_fields.Nested(lambda: NRDegreeGrantorSchema()))

    studyFields = ma_fields.List(ma_fields.String())


class KeywordsSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE


class NRDegreeGrantorSchema(DictOnlySchema):
    class Meta:
        unknown = ma.INCLUDE

    _id = String(data_key="id", attribute="id")

    _version = String(data_key="@v", attribute="@v")

    hierarchy = ma_fields.Nested(lambda: HierarchySchema())

    title = i18n_strings


class NRDocumentSyntheticFieldsSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    organizations = ma_fields.String()



from oarepo_runtime.records.dumpers import SearchDumper
from oarepo_runtime.records.systemfields.mapping import SystemFieldDumperExt

from nr_metadata.datacite.records.dumpers.edtf import DataciteEDTFIntervalDumperExt
from nr_metadata.datacite.records.dumpers.multilingual import (
    MultilingualSearchDumperExt,
)


class DataciteDumper(SearchDumper):
    """DataciteRecord opensearch dumper."""

    extensions = [
        SystemFieldDumperExt(),
        DataciteEDTFIntervalDumperExt(),
        MultilingualSearchDumperExt(),
    ]



import marshmallow as ma
from edtf import Date as EDTFDate
from invenio_rdm_records.services.schemas.access import AccessSchema
from invenio_rdm_records.services.schemas.pids import PIDSchema
from invenio_rdm_records.services.schemas.record import validate_scheme
from marshmallow import Schema
from marshmallow import fields as ma_fields
from marshmallow.fields import Dict, Nested
from marshmallow.validate import OneOf
from marshmallow_utils.fields import SanitizedUnicode, TrimmedString
from marshmallow_utils.fields.nestedattr import NestedAttribute
from oarepo_runtime.services.schema.i18n import I18nStrField, MultilingualField
from oarepo_runtime.services.schema.marshmallow import (
    DictOnlySchema,
    RDMBaseRecordSchema,
)
from oarepo_runtime.services.schema.rdm import FundingSchema
from oarepo_runtime.services.schema.validation import (
    CachedMultilayerEDTFValidator,
    validate_date,
    validate_datetime,
    validate_identifier,
)

from nr_metadata.common.services.records.schema_datatypes import (
    NREventSchema,
    NRGeoLocationSchema,
    NRLanguageVocabularySchema,
    NRRelatedItemSchema,
    NRResourceTypeVocabularySchema,
    NRRightsVocabularySchema,
    NRSeriesSchema,
    NRSubjectCategoryVocabularySchema,
    NRSubjectSchema,
)
from nr_metadata.schema.creators import RDMNTKCreatorsSchema
from nr_metadata.schema.identifiers import (
    NRObjectIdentifierSchema,
    NRSystemIdentifierSchema,
)


class NRCommonRecordSchema(RDMBaseRecordSchema):
    class Meta:
        unknown = ma.RAISE

    access = NestedAttribute(lambda: AccessSchema())

    metadata = ma_fields.Nested(lambda: NRCommonMetadataSchema())

    pids = Dict(
        keys=SanitizedUnicode(validate=validate_scheme),
        values=Nested(PIDSchema),
    )

    state = ma_fields.String(dump_only=True)

    state_timestamp = ma_fields.String(dump_only=True, validate=[validate_datetime])


class NRCommonMetadataSchema(Schema):
    class Meta:
        unknown = ma.RAISE

    abstract = MultilingualField(I18nStrField())

    accessibility = MultilingualField(I18nStrField())

    additionalTitles = ma_fields.List(
        ma_fields.Nested(lambda: AdditionalTitlesSchema())
    )

    contributors = ma_fields.List(ma_fields.Nested(lambda: RDMNTKCreatorsSchema()))

    creators = ma_fields.List(
        ma_fields.Nested(lambda: RDMNTKCreatorsSchema()),
        required=True,
        validate=[ma.validate.Length(min=1)],
    )

    dateAvailable = ma_fields.String(validate=[validate_date("%Y-%m-%d")])

    dateIssued = TrimmedString(
        required=True, validate=[CachedMultilayerEDTFValidator(types=(EDTFDate,))]
    )

    events = ma_fields.List(ma_fields.Nested(lambda: NREventSchema()))

    funders = ma_fields.List(ma_fields.Nested(lambda: FundingSchema()))

    geoLocations = ma_fields.List(ma_fields.Nested(lambda: NRGeoLocationSchema()))

    languages = ma_fields.List(ma_fields.Nested(lambda: NRLanguageVocabularySchema()))

    methods = MultilingualField(I18nStrField())

    notes = ma_fields.List(ma_fields.String())

    objectIdentifiers = ma_fields.List(
        ma_fields.Nested(
            lambda: NRObjectIdentifierSchema(),
            validate=[lambda value: validate_identifier(value)],
        )
    )

    originalRecord = ma_fields.String()

    relatedItems = ma_fields.List(ma_fields.Nested(lambda: NRRelatedItemSchema()))

    resourceType = ma_fields.Nested(
        lambda: NRResourceTypeVocabularySchema(), required=True
    )

    rights = ma_fields.Nested(lambda: NRRightsVocabularySchema())

    series = ma_fields.List(ma_fields.Nested(lambda: NRSeriesSchema()))

    subjectCategories = ma_fields.List(
        ma_fields.Nested(lambda: NRSubjectCategoryVocabularySchema())
    )

    subjects = ma_fields.List(ma_fields.Nested(lambda: NRSubjectSchema()))

    systemIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: NRSystemIdentifierSchema())
    )

    technicalInfo = MultilingualField(I18nStrField())

    title = ma_fields.String(required=True)

    version = ma_fields.String()


class AdditionalTitlesSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    title = I18nStrField(required=True)

    titleType = ma_fields.String(
        required=True,
        validate=[OneOf(["translatedTitle", "alternativeTitle", "subtitle", "other"])],
    )
